import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

public class Debugger extends List implements CommandListener
{
    PCalc    calc;
    Launcher parent;
    VirtualMachine vm;
    Procedure curr;
    int       selectedIndex;

    Debugger(PCalc calc, Launcher parent, Program prog) {
        super("Debugger", IMPLICIT);
        this.calc = calc;
        this.parent = parent;

        vm = new VirtualMachine(prog);

        curr = prog.main;
        String[] insns = VirtualMachine.printInstructions(curr);
        for (int i = 0; i < insns.length; i++) { 
            append(insns[i], null);
        }
        
        setCommandListener(this);
        addCommand(PCalc.STEP_CMD);
        addCommand(PCalc.CONTINUE_CMD);           
        addCommand(PCalc.RESTART_CMD);           
        addCommand(PCalc.VARIABLES_CMD);           
        addCommand(PCalc.STACK_CMD);           
        addCommand(PCalc.TRACE_CMD);           
        addCommand(PCalc.CANCEL_CMD);
        Display.getDisplay(calc).setCurrent(this);
    }

    Debugger(PCalc calc, Launcher parent, VirtualMachine vm, boolean recoverable) {
        super("Debugger", IMPLICIT);
        this.calc = calc;
        this.parent = parent;
        this.vm = vm;
        
        curr = vm.currFrame.proc;
        String[] insns = VirtualMachine.printInstructions(curr);
        for (int i = 0; i < insns.length; i++) { 
            append(insns[i], null);
        }
        setSelectedIndex(VirtualMachine.getInstructionForPc(curr.code, vm.currFrame.pc), true);
         
        setCommandListener(this);
        if (recoverable) { 
            addCommand(PCalc.STEP_CMD);
            addCommand(PCalc.CONTINUE_CMD);           
        }
        addCommand(PCalc.RESTART_CMD);           
        addCommand(PCalc.VARIABLES_CMD);           
        addCommand(PCalc.STACK_CMD);           
        addCommand(PCalc.TRACE_CMD);           
        addCommand(PCalc.CANCEL_CMD);
        Display.getDisplay(calc).setCurrent(this);
    }

    void goBack() { 
        setSelectedIndex(selectedIndex, true);
        Display.getDisplay(calc).setCurrent(this);
    }       

    public void commandAction(Command c, Displayable d) 
    {
        selectedIndex = getSelectedIndex();
        if (c == PCalc.CANCEL_CMD) { 
            Display.getDisplay(calc).setCurrent(parent.parent);
        } else if (c == PCalc.STEP_CMD) {
            if (vm.cSp == 0) { 
                 calc.showError("Program is terminated", this);
                 return;
            }               
            try { 
                vm.step();
            } catch (ArithmeticException x) { 
                calc.showError("Arithmetic exception", this);
                return;
            } catch (StackEmptyException x) {
                calc.showError("Stack empty", this);
                return;
            } catch (StackOverflowException x) {
                calc.showError("Stack overflow", this);
                return;
            }
            if (vm.currFrame.proc != curr) { 
                while (size() > 0) { 
                    delete(size()-1);
                }
                curr = vm.currFrame.proc;
                String[] insns = VirtualMachine.printInstructions(curr);
                for (int i = 0; i < insns.length; i++) { 
                    append(insns[i], null);
                }
            }
            setSelectedIndex(VirtualMachine.getInstructionForPc(curr.code, vm.currFrame.pc), true);
        } else if (c == PCalc.CONTINUE_CMD) { 
            new Monitor(calc, parent, vm);
        } else if (c == PCalc.RESTART_CMD) {
            Display.getDisplay(calc).setCurrent(parent);
        } else if (c == PCalc.VARIABLES_CMD) {
            new InspectVariables(calc, this, vm.currFrame);
        } else if (c == PCalc.STACK_CMD) { 
            new InspectStack(calc, this, vm);
        } else if (c == PCalc.TRACE_CMD) { 
            new BackTrace(calc, this, vm);
        }
    }
}
           


