import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

public class EditProcedure extends List implements CommandListener 
{
    PCalc       calc;
    Displayable parent;
    Procedure   proc;

    EditProcedure(PCalc calc, Displayable parent, Procedure proc) { 
        super(proc.name, List.IMPLICIT);
        this.calc = calc;
        this.parent = parent;
        this.proc = proc;

        String[] insns = VirtualMachine.printInstructions(proc);
        for (int i = 0; i < insns.length; i++) { 
            append(insns[i], null);
        }

        setCommandListener(this);
        addCommand(PCalc.INSERT_CMD);
        addCommand(PCalc.EDIT_CMD);
        addCommand(PCalc.DELETE_CMD);
        addCommand(PCalc.FUNCTIONS_CMD);
        addCommand(PCalc.VARIABLES_CMD);
        addCommand(PCalc.MOVE_CMD);
        addCommand(PCalc.SAVE_CMD);
        Display.getDisplay(calc).setCurrent(this);
    }

    void redraw() 
    { 
        int pos = getSelectedIndex();
        while (size() > 0) { 
            delete(size()-1);
        }
        String[] insns = VirtualMachine.printInstructions(proc);
        for (int i = 0; i < insns.length; i++) { 
            append(insns[i], null);
        }
        setSelectedIndex(pos, true);
    }
        

    public void commandAction(Command c, Displayable d) 
    {
        int i = getSelectedIndex();
        if (c == PCalc.SAVE_CMD) { 
            Display.getDisplay(calc).setCurrent(parent);
        } else if (c == PCalc.INSERT_CMD) { 
            if (i < size()-1) { 
                int pc = VirtualMachine.getPcForInstruction(proc.code, i);
                new SelectCommandGroup(calc, this, proc, i, pc);
            }
        } else if (c == PCalc.DELETE_CMD) { 
            if (i < size()-1) { 
                int pc = VirtualMachine.getPcForInstruction(proc.code, i);
                if (i == size()-2) { 
                    calc.showError("Can not remove last intruction", this);
                } else { 
                    delete(i);
                    proc.removeInstruction(pc);
                }                
            }
        } else if (c == PCalc.FUNCTIONS_CMD) { 
            new ProcedureList(calc, parent, proc);
        } else if (c == PCalc.VARIABLES_CMD) { 
            new VariableList(calc, this, proc);
        } else if (c == PCalc.MOVE_CMD) { 
             if (i < size()-1) { 
                int pc = VirtualMachine.getPcForInstruction(proc.code, i);
                String cmd = getString(i);
                if (cmd.indexOf(':') >= 0) { 
                    new MoveLabel(calc, this, proc, i, pc);
                } else { 
                    calc.showError("Instruction has no label", this);
                }
             }
        } else if (i < size()-1) { 
            int pc = VirtualMachine.getPcForInstruction(proc.code, i);
            switch (proc.code[pc]) {
            case VirtualMachine.LOAD:
            case VirtualMachine.STORE:
                new ChooseVariable(calc, this, proc, i, pc, proc.code[pc], true);
                break;
            case VirtualMachine.LITERAL:
                new InputLiteral(calc, this, proc, i, pc, true);
                break;
            case VirtualMachine.INPUT:
                new ChooseInputVariable(calc, this, proc, i, pc, true);
                break;
            case VirtualMachine.CALL:
                new ChooseFunction(calc, this, proc, i, pc, true);
                break;
            case VirtualMachine.IF:
            case VirtualMachine.GOTO:
                new SetLabel(calc, this, proc, i, pc, proc.code[pc], true);
                break;
            default:
                calc.showError("Non-editable instruction", this);
            }
        }
    }
}
