import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import java.util.Vector;

public class PCalc extends MIDlet implements CommandListener  
{ 
    static Command NEW_CMD = new Command("New", Command.SCREEN, 1);
    static Command ADD_CMD = new Command("Add", Command.SCREEN, 1);
    static Command INSERT_CMD = new Command("Insert", Command.SCREEN, 1);
    static Command SAVE_CMD = new Command("Save", Command.SCREEN, 2);
    static Command CANCEL_CMD = new Command("Cancel", Command.BACK, 2);
    static Command OK_CMD = new Command("Ok", Command.SCREEN, 1);
    static Command DONE_CMD = new Command("Done", Command.SCREEN, 2);
    static Command BACK_CMD = new Command("Back", Command.BACK, 1);
    static Command EDIT_CODE_CMD = new Command("Code", Command.SCREEN, 1);
    static Command RUN_CMD = new Command("Run", Command.ITEM, 2);
    static Command DEBUG_CMD = new Command("Debug", Command.ITEM, 2);
    static Command EDIT_CMD = new Command("Edit", Command.ITEM, 2);
    static Command RENAME_CMD = new Command("Rename", Command.ITEM, 2);
    static Command DELETE_CMD = new Command("Delete", Command.ITEM, 2);
    static Command UNDELETE_CMD = new Command("Undelete", Command.SCREEN, 2);
    static Command EXIT_CMD = new Command("Exit", Command.SCREEN, 2);
    static Command STEP_CMD = new Command("Step", Command.SCREEN, 1);
    static Command RESTART_CMD = new Command("Restart", Command.SCREEN, 2);
    static Command START_CMD = new Command("Start", Command.SCREEN, 1);
    static Command STOP_CMD = new Command("Stop", Command.SCREEN, 1);
    static Command CONTINUE_CMD = new Command("Continue", Command.SCREEN, 2);
    static Command FUNCTIONS_CMD = new Command("Functions", Command.SCREEN, 2);
    static Command VARIABLES_CMD = new Command("Variables", Command.SCREEN, 2);
    static Command PARAMETERS_CMD = new Command("Parameters", Command.SCREEN, 2);
    static Command STACK_CMD = new Command("Stack", Command.SCREEN, 2);
    static Command TRACE_CMD = new Command("Back trace", Command.SCREEN, 2);
    static Command MOVE_CMD = new Command("Move label", Command.SCREEN, 2);

    
    static final String STORE_NAME = "PCALC.002";
   
    List progList;
    RecordStore store;
    Program deleted;
    Program edited;
    Vector programs;

    public PCalc() { 
        progList = new List("Programs", List.IMPLICIT);
        progList.addCommand(NEW_CMD);
        progList.addCommand(RUN_CMD);
        progList.addCommand(DEBUG_CMD);
        progList.addCommand(EDIT_CMD);
        progList.addCommand(DELETE_CMD);
        progList.addCommand(UNDELETE_CMD);
        progList.addCommand(EXIT_CMD);
    }

    protected void destroyApp(boolean unconditional) 
    {
        if (edited != null) { 
            saveProgram(edited);
        }
        progList = null;
        closeStore();
    }    

    protected  void pauseApp() 
    { 
    }

    protected void startApp() 
    {    
        Display display = Display.getDisplay(this);
	progList.setCommandListener(this);
        display.setCurrent(progList);
        if (store == null) { 
            openStore();
        }
    }


    private void openStore() {
        try { 
            store = RecordStore.openRecordStore(STORE_NAME, true);
        } catch(Exception x) { 
        }
        loadPrograms();
    }

    private void closeStore() 
    {
        if (store != null) { 
            try { 
                store.closeRecordStore();
                store = null;
            } catch (Exception x) {}
        }
    }

    void loadPrograms() { 
        programs = new Vector();        
        try { 
            for (int recordId = store.getNextRecordID(); --recordId > 0;) { 
                byte[] record = null;
                try { 
                    record = store.getRecord(recordId);                    
                } catch (InvalidRecordIDException x) {
                    // record was deleted 
                }
                if (record == null) { 
                    continue;
                }
                Program p = new Program(recordId, record);
                programs.addElement(p);
                progList.append(p.name, null);
            }
        } catch (Exception x) {
            x.printStackTrace();
            showError("Failed to load programs: " + x, progList);
        }
    }
            
    void saveProgram(Program p) {
        try { 
            byte[] record = p.pack();
            if (p.recordId == 0) { 
                p.recordId = store.addRecord(record, 0, record.length);
            } else { 
                store.setRecord(p.recordId, record, 0, record.length);
            }
        } catch (Exception x) {}
        edited = null;
    }

    
    void removeProgram(Program p, int i) {        
        try { 
            store.deleteRecord(p.recordId);
        } catch (Exception x) {}
        progList.delete(i);
        programs.removeElementAt(i);
        p.recordId = 0;
        edited = null;
    }

    void showError(String msg, Displayable parent) { 
        Alert a = new Alert("Error", msg, null, AlertType.ERROR);
        a.setTimeout(Alert.FOREVER);
        Display.getDisplay(this).setCurrent(a, parent);
    }        
    
    public void commandAction(Command c, Displayable d) 
    {
        int i = progList.getSelectedIndex();
        if (c == NEW_CMD) {
            edited = new Program("Program " + (programs.size()+1));
            programs.insertElementAt(edited, 0);
            progList.insert(0, edited.name, null);
            new EditProcedure(this, progList, edited.main); 
        } else if (c == RUN_CMD) { 
            if (i >= 0) { 
                new Launcher(this, progList, (Program)programs.elementAt(i), false);
            }
        } else if (c == DEBUG_CMD) { 
            if (i >= 0) { 
                new Launcher(this, progList, (Program)programs.elementAt(i), true);
            }
        } else if (c == DELETE_CMD) { 
            if (i >= 0) { 
                deleted = (Program)programs.elementAt(i);
                removeProgram(deleted, i);
            }
        } else if (c == UNDELETE_CMD) { 
            if (deleted == null) { 
                showError("No program was deleted", progList);
            } else {
                programs.insertElementAt(deleted, 0);
                progList.insert(0, deleted.name, null);
                saveProgram(deleted);
                deleted = null;
            }
        } else if (c == EXIT_CMD) {
            destroyApp(true);
            notifyDestroyed();
        } else { 
            if (i >= 0) { 
                edited = (Program)programs.elementAt(i);
                new EditProgram(this, progList, i, edited);
            }
        }
    }     
}
        
