public class Program { 
    int       recordId;
    String    name;
    String    description;
    Procedure main;
    Float[]   inputParams;
    String[]  inputParamNames;

    static final int MAX_PARAMETERS = 0x100;
    
    byte[] pack() {
        ByteBuffer buf = new ByteBuffer();        
        int offs = 0;
        byte[] nameBytes = Converter.packString(name);        
        buf.extend(1 + nameBytes.length);
        buf.arr[offs++] = (byte)nameBytes.length;
        System.arraycopy(nameBytes, 0, buf.arr, offs, nameBytes.length);
        offs += nameBytes.length;
        
        byte[] descBytes = Converter.packString(description);
        buf.extend(offs + 1 + descBytes.length);
        buf.arr[offs++] = (byte)descBytes.length;
        System.arraycopy(descBytes, 0, buf.arr, offs, descBytes.length);
        offs += descBytes.length;
        
        buf.extend(offs + 1 + VirtualMachine.SIZEOF_FLOAT*inputParams.length);
        buf.arr[offs++] = (byte)inputParams.length;
        for (int i = 0; i < inputParams.length; i++) { 
            VirtualMachine.packFloat(buf.arr, offs, inputParams[i]);
            offs += VirtualMachine.SIZEOF_FLOAT;
        }
        for (int i = 0; i < inputParamNames.length; i++) { 
            byte[] paramNameBytes = Converter.packString(inputParamNames[i]);
            buf.extend(offs + 1 + 4*paramNameBytes.length);
            buf.arr[offs++] = (byte)paramNameBytes.length;
            System.arraycopy(paramNameBytes, 0, buf.arr, offs, paramNameBytes.length);
            offs += paramNameBytes.length;
        }        
        main.pack(buf, offs);
        return buf.toArray();
    }
    
    void unpack(byte[] record) { 
        int offs = 0;
        name = Converter.unpackString(record, offs + 1, record[offs] & 0xFF); 
        offs += 1 + (record[offs] & 0xFF);
        description = Converter.unpackString(record, offs + 1, record[offs] & 0xFF); 
        offs += 1 + (record[offs] & 0xFF);

        inputParams = new Float[record[offs++] & 0xFF];
        for (int i = 0; i < inputParams.length; i++) { 
            inputParams[i] = VirtualMachine.unpackFloat(record, offs);
            offs += VirtualMachine.SIZEOF_FLOAT;
        }
        inputParamNames = new String[inputParams.length];
        for (int i = 0; i < inputParamNames.length; i++) { 
            inputParamNames[i] = Converter.unpackString(record, offs + 1, record[offs] & 0xFF); 
            offs += 1 + (record[offs] & 0xFF);
        }

        main = new Procedure(this, null);
        main.unpack(record, offs);
    }

    int findInputParam(String name) { 
        for (int i = 0; i < inputParamNames.length; i++) { 
            if (inputParamNames[i].equals(name)) { 
                return i;
            }
        }
        return -1;
    }

    int addInputParam(String name) { 
        if (findInputParam(name) >= 0) { 
            return -1;
        }
        int n = inputParams.length;
        Float[] newInputParams = new Float[n+1];
        String[] newInputParamNames = new String[n+1];
        System.arraycopy(inputParams, 0, newInputParams, 0, n);
        System.arraycopy(inputParamNames, 0, newInputParamNames, 0, n);
        inputParams = newInputParams;
        inputParamNames = newInputParamNames;        
        inputParamNames[n] = name;
        inputParams[n] = new Float();
        return n;
    }

    
    boolean removeInputParam(int i) { 
        if (main.findParameterAccess(i)) { 
            return false;
        }
        int n = inputParams.length;
        Float[] newInputParams = new Float[n-1];
        String[] newInputParamNames = new String[n-1];
        System.arraycopy(inputParams, 0, newInputParams, i, i);
        System.arraycopy(inputParamNames, 0, newInputParamNames, i, i);
        System.arraycopy(inputParams, i+1, newInputParams, i, n-i-1);
        System.arraycopy(inputParamNames, i+1, newInputParamNames, 0, n-i-1);
        inputParams = newInputParams;
        inputParamNames = newInputParamNames; 
        main.adjustParameterAccess(i);
        return true;
    }

    
    Program(String name) {
        this.name = name;
        this.description = "No description";
        main = new Procedure(this, null, "main", 0);
        inputParams = new Float[0];
        inputParamNames = new String[0];
    }

    Program(int recordId, byte[] record) { 
        this.recordId = recordId;
        unpack(record);
    }
}
